#include <mpi.h>

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define SIZE 1000000

void read_nums(int* array)
{
	//File containing SIZE numbers
	FILE* file;

	//Loop variable
	int i;

	file = fopen("input.dat", "r");
	for (i=0; i < SIZE; ++i) {
		fscanf(file, "%d", &array[i]);
	}
	fclose(file);
}

int do_sum(int* array, int count)
{
	int i;
	int sum = 0;
	for (i=0; i < count; ++i) {
		sum += array[i];
	}
	return sum;
}

int main(int argc, char* argv[])
{
	//Array to store the numbers in
	int nums[SIZE];

	//Array to store smaller chunk of numbers
	int* temp;

	//Number of processes
 	int np; 

	//Process number of THIS process 
	int p; 

	//Result
	int sum = 0;
	int totalsum = 0;

	//Set up MPI
	MPI_Init(&argc, &argv);

	//Fetch number of processes
        MPI_Comm_size(MPI_COMM_WORLD, &np);
	
	//Fetch process number of THIS process
	MPI_Comm_rank(MPI_COMM_WORLD, &p) ;

	//On only the root node, read in the numbers
	if (p==0) {
		read_nums(nums);
	}

	//Send a block of SIZE/np numbers to each process
	temp = calloc(SIZE/np, sizeof(int));
	MPI_Scatter(nums, SIZE/np, MPI_INT, temp, SIZE/np, MPI_INT, 0, MPI_COMM_WORLD); 

	//Compute sum of -our- chunk
	sum = do_sum(temp, SIZE/np);

	//Add all the chunks together as we send them back to the root node.
	MPI_Reduce(&sum, &totalsum, 1, MPI_INT, MPI_SUM, 0, MPI_COMM_WORLD);

	//Wait for all threads to finish
	MPI_Barrier(MPI_COMM_WORLD);

	//Output the average
	if (p==0) {
		printf("%lf\n", (totalsum*1.0)/SIZE);
	}

	//Clean up MPI data structures
	MPI_Finalize();
}
